@@ -237,23 +237,35 @@ public:
 	private:
 		friend class Cipher;
 	
-		const void *algotype;
-		const void *hashtype;
+		union {
+			const void *algotype;
+			int algoid;
+		};
+
+		union {
+			const void *hashtype;
+			int hashid;
+		};
+
+		int modeid;
 		
 		// assume 512 bit cipher keys possible...
 		unsigned char keybuf[MAX_CIPHER_KEYSIZE / 8], ivbuf[MAX_CIPHER_KEYSIZE / 8];
 
 		// generated keysize
-		unsigned keysize, blksize;
+		size_t keysize, blksize;
 
 	public:
 		Key(const char *cipher, const char *digest, const char *text, size_t size = 0, const unsigned char *salt = NULL, unsigned rounds = 1);
 		Key();
+		~Key();
 
-		inline unsigned size(void)
+		void clear(void);
+
+		inline size_t size(void)
 			{return keysize;};
 
-		inline unsigned iosize(void)
+		inline size_t iosize(void)
 			{return blksize;};
 	};
 
@@ -274,17 +286,60 @@ protected:
 public:
 	Cipher();
 
-	Cipher(key_t key, mode_t mode, unsigned char *address, size_t size = 0);
+	Cipher(key_t key, mode_t mode, unsigned char *address = NULL, size_t size = 0);
 
 	~Cipher();
 
+	void set(unsigned char *address, size_t size = 0);
+
 	void set(key_t key, mode_t mode, unsigned char *address, size_t size = 0);
 
+	/**
+	 * Push a final cipher block.  This is used to push the final buffer into
+	 * the push method for any remaining data.
+	 */
 	size_t flush(void);
 
+	/**
+	 * Process cipher data.  This requires the size to be a multiple of the
+	 * cipher block size.  If an unaligned sized block of data is used, it
+	 * will be ignored and the size returned will be 0.
+	 * @param data to process.
+	 * @param size of data to process.
+	 * @return size of processed output, should be same as size or 0 if error.
+	 */
 	size_t put(const unsigned char *data, size_t size);
 
-	size_t puts(const char *str);
+	/**
+	 * This essentially encrypts a single string and pads with NULL bytes
+	 * as needed.
+	 * @param string to encrypt.
+	 * @return total encrypted size.
+	 */
+	size_t puts(const char *string);
+
+	/**
+	 * This is used to process any data unaligned to the blocksize at the end
+	 * of a cipher session.  On an encryption, it will add padding or an
+	 * entire padding block with the number of bytes to strip.  On decryption
+	 * it will remove padding at the end.  The pkcs5 method of padding with
+	 * removal count is used.  This also sets the address buffer to NULL
+	 * to prevent further puts until reset.
+	 * @param address of data to add before final pad.
+	 * @param size of data to add before final pad.
+	 * @return actual bytes encrypted or decrypted.
+	 */
+	size_t pad(const unsigned char *data, size_t size);
+
+	/**
+	 * Process encrypted data in-place.  This assumes no need to set the
+	 * address buffer.
+	 * @param address of data to process.
+	 * @param size of data to process.
+	 * @param flag if to pad data.
+	 * @return bytes processed and written back to buffer.
+	 */
+	size_t process(unsigned char *data, size_t size, bool flag = false);
 		
 	inline size_t size(void)
 		{return bufsize;};
@@ -292,7 +347,15 @@ public:
 	inline size_t pos(void)
 		{return bufpos;};
 
-	size_t alignment(void);
+	inline size_t align(void)
+		{return keys.iosize();};
+
+	/**
+	 * Check if a specific cipher is supported.
+	 * @param name of cipher to check.
+	 * @return true if supported, false if not.
+	 */
+	static bool is(const char *name);
 };
 
 /**
@@ -305,7 +368,12 @@ class __EXPORT Digest
 {
 private:
 	void *context;
-	const void *hashtype;
+
+	union {
+		const void *hashtype;
+		int hashid;
+	};
+
 	unsigned bufsize;
 	unsigned char buffer[MAX_DIGEST_HASHSIZE / 8];
 	char text[MAX_DIGEST_HASHSIZE / 8 + 1];
@@ -351,6 +419,13 @@ public:
 
 	inline operator bool() const
 		{return bufsize > 0 || context != NULL;};
+
+	/**
+	 * Test to see if a specific digest type is supported.
+	 * @param name of digest we want to check.
+	 * @return true if supported, false if not.
+	 */
+	static bool is(const char *name);
 };
 
 /**
@@ -450,6 +525,11 @@ typedef	Cipher cipher_t;
  */
 typedef Cipher::Key skey_t;
 
+inline void zerofill(void *addr, size_t size)
+{
+	::memset(addr, 0, size);
+}
+
 END_NAMESPACE
 
 #endif
